@@ -1,18 +1,22 @@
 from django.conf.urls.defaults import patterns, url
 from django.utils.functional import update_wrapper
 from django.utils.encoding import force_unicode
+from django.utils.translation import ugettext as _
 from django.core.urlresolvers import reverse
 from django.contrib.admin import widgets
 from django.contrib.admin.options import get_ul_class
+from django.contrib.contenttypes.models import ContentType
 from django import forms
 
 from dockit.paginator import Paginator
 from dockit.forms import DocumentForm
-from dockit.forms.fields import PrimitiveListField
+from dockit.forms.fields import PrimitiveListField, HiddenJSONField
+from dockit.models import DockitPermission
 
 import views
 from widgets import AdminPrimitiveListWidget
 from breadcrumbs import Breadcrumb
+from objecttools import LinkObjectTool
 
 FORMFIELD_FOR_FIELD_DEFAULTS = {
     forms.DateTimeField: {
@@ -35,6 +39,7 @@ class SchemaAdmin(object):
     create = views.CreateView
     update = views.UpdateView
     delete = views.DeleteView
+    history = views.HistoryView
     select_schema = views.SchemaTypeSelectionView
     field_list_index = views.ListFieldIndexView
     
@@ -63,6 +68,7 @@ class SchemaAdmin(object):
     list_filter = ()
     list_select_related = False
     list_per_page = 100
+    list_max_show_all = 200
     list_editable = ()
     search_fields = ()
     date_hierarchy = None
@@ -77,11 +83,14 @@ class SchemaAdmin(object):
     object_history_template = None
     
     schema = None
+    
+    proxy_django_model = DockitPermission #if we really need to give a model to django, let it be this one
 
     def __init__(self, model, admin_site, schema=None, documentadmin=None):
         self.model = model
         self.admin_site = admin_site
         self.app_name = model._meta.app_label +'_'+ model._meta.module_name
+        self.app_label = model._meta.app_label
         overrides = FORMFIELD_FOR_FIELD_DEFAULTS.copy()
         overrides.update(self.formfield_overrides)
         self.formfield_overrides = overrides
@@ -106,7 +115,7 @@ class SchemaAdmin(object):
         #if self.opts.get_ordered_objects():
         #    js.extend(['js/getElementsBySelector.js', 'js/dom-drag.js' , 'js/admin/ordering.js'])
 
-        return forms.Media(js=['%s%s' % (settings.ADMIN_MEDIA_PREFIX, url_s) for url_s in js])
+        return forms.Media(js=['%s%s' % (getattr(settings, 'ADMIN_MEDIA_PREFIX', 'admin'), url_s) for url_s in js])
     media = property(_media)
     
     def has_add_permission(self, request):
@@ -207,10 +216,10 @@ class SchemaAdmin(object):
         
         return inline_instances
     
-    def get_formsets(self, request, obj=None):
+    def get_formsets(self, request, view, obj=None):
         inline_instances = self.get_inline_instances()
         for inline in inline_instances:
-            yield inline.get_formset(request, obj)
+            yield inline.get_formset(request, view=view, obj=obj)
     
     def get_fieldsets(self, request, obj=None):
         "Hook for specifying fieldsets for the add form."
@@ -219,15 +228,17 @@ class SchemaAdmin(object):
         #form = self.get_form(request, obj)
         #fields = form.base_fields.keys() + list(self.get_readonly_fields(request, obj))
         fields = list()
+        excludes = self.get_excludes()
         for key, field in self.schema._meta.fields.iteritems():
-            fields.append(key) #TODO handle exclude
+            if key in excludes:
+                continue
+            if not field.editable:
+                continue
+            fields.append(key)
         return [(None, {'fields': fields})]
     
     def formfield_for_field(self, prop, field, view, **kwargs):
         from dockit import schema
-        from fields import DotPathField
-        from dockit.forms.fields import HiddenJSONField
-        
         if isinstance(prop, schema.ModelReferenceField):
             return self.formfield_for_foreignkey(prop, field, view, **kwargs)
         if isinstance(prop, schema.ModelSetField):
@@ -261,16 +272,11 @@ class SchemaAdmin(object):
                 kwargs['required'] = False
             return field(**kwargs)
         if issubclass(field, HiddenJSONField):
-            field = DotPathField
-            kwargs['dotpath'] = view.dotpath()
-            kwargs['params'] = request.GET.copy()
-            if view.next_dotpath():
-                kwargs['required'] = False
-            return field(**kwargs)
+            return self.formfield_for_jsonfield(prop, field, view, request=request, **kwargs)
         if issubclass(field, PrimitiveListField) and 'subfield' in kwargs:
-            subfield_kwargs = dict(kwargs)
+            subfield_kwargs = prop.subfield.formfield_kwargs()
             subfield_kwargs.pop('initial', None)
-            subfield = self.formfield_for_field(prop, type(subfield_kwargs.pop('subfield')), view, **subfield_kwargs)
+            subfield = self.formfield_for_field(prop, type(kwargs['subfield']), view, **subfield_kwargs)
             kwargs['subfield'] = subfield
             kwargs['widget'] = AdminPrimitiveListWidget
         if field in self.formfield_overrides:
@@ -279,13 +285,43 @@ class SchemaAdmin(object):
             kwargs = dict(opts, **kwargs)
         return field(**kwargs)
     
+    def formfield_for_jsonfield(self, prop, field, view, **kwargs):
+        request = kwargs.pop('request', None)
+        from fields import DotPathField
+        field = DotPathField
+        kwargs['dotpath'] = view.dotpath()
+        kwargs['params'] = request.GET.copy()
+        if view.next_dotpath():
+            kwargs['required'] = False
+        return field(**kwargs)
+
+    def raw_id_widget_for_property(self,prop,WidgetType):
+        class FauxRel(object):
+            def get_related_field(self):
+                return self   
+        rel = FauxRel()
+        rel.to = prop.model
+        rel.limit_choices_to = {}
+        rel.name = "pk" 
+        thewidget = WidgetType(rel=rel)
+        # Monkey patch the render function to replace
+        # the hardcoded admin relative url
+        thewidget._render = thewidget.render
+        def _tmprender(name,value,attrs=None):
+            from django.utils.safestring import mark_safe
+            from django.core.urlresolvers import reverse  
+            output = thewidget._render(name,value,attrs=attrs) 
+            return mark_safe(output.replace("../../../",reverse("admin:index")))
+        thewidget.render = _tmprender
+        return thewidget
+
     def formfield_for_foreignkey(self, prop, field, view, **kwargs):
         """
         Get a form Field for a ForeignKey.
         """
         request = kwargs.pop('request', None)
-        if prop.name in self.raw_id_fields:
-            kwargs['widget'] = widgets.ForeignKeyRawIdWidget(rel=None)
+        if True: #prop.name in self.raw_id_fields:
+            kwargs["widget"] = self.raw_id_widget_for_property(prop,widgets.ForeignKeyRawIdWidget)
         elif prop.name in self.radio_fields:
             kwargs['widget'] = widgets.AdminRadioSelect(attrs={
                 'class': get_ul_class(self.radio_fields[prop.name]),
@@ -299,8 +335,8 @@ class SchemaAdmin(object):
         Get a form Field for a ManyToManyField.
         """
         request = kwargs.pop('request', None)
-        if prop.name in self.raw_id_fields:
-            kwargs['widget'] = widgets.ManyToManyRawIdWidget(rel=None)
+        if True: #prop.name in self.raw_id_fields:
+            kwargs['widget'] = self.raw_id_widget_for_property(prop,widgets.ManyToManyRawIdWidget)
             kwargs['help_text'] = ''
         elif prop.name in (list(self.filter_vertical) + list(self.filter_horizontal)):
             kwargs['widget'] = widgets.FilteredSelectMultiple(prop.verbose_name, (prop.name in self.filter_vertical))
@@ -330,9 +366,18 @@ class SchemaAdmin(object):
     def get_paginator(self, request, query_set, paginate_by):
         return self.paginator(query_set, paginate_by)
     
-    def get_object_tools(self, request, object=None):
-        #TODO object tools are renderable object that are displayed at the top the admin
-        #TODO return history object tool if there is an object
+    def get_object_tools(self, request, object=None, add=False):
+        #object tools are renderable object that are displayed at the top the admin
+        init = self.get_view_kwargs()
+        if object:
+            return [self.history(**init).get_object_tool(request, object)]
+        elif add:
+            if self.schema:
+                opts = self.schema._meta
+            else:
+                opts = self.model._meta
+            title = _('Add %s') % force_unicode(opts.verbose_name)
+            return [LinkObjectTool(title, self.reverse('%s_add' % self.app_name), css_class='addlink')]
         return []
     
     def get_field(self, schema, dotpath, obj=None):
@@ -355,11 +400,11 @@ class SchemaAdmin(object):
                 assert field
         return field
     
-    def get_base_breadcrumbs(self):
-        return self.documentadmin.get_base_breadcrumbs()
+    def get_base_breadcrumbs(self, top=False):
+        return self.documentadmin.get_base_breadcrumbs(top)
     
-    def get_instance_breadcrumb(self, obj=None):
-        return self.documentadmin.get_instance_breadcrumb(obj)
+    def get_instance_breadcrumb(self, obj=None, include_link=False):
+        return self.documentadmin.get_instance_breadcrumb(obj, include_link)
 
 class DocumentAdmin(SchemaAdmin):
     # Actions
@@ -375,7 +420,6 @@ class DocumentAdmin(SchemaAdmin):
     
     delete = views.DeleteView
     index = views.IndexView
-    history = views.HistoryView
     detail_views = [views.HistoryView, views.DeleteView]
     default_schema_admin = SchemaAdmin
     schema_inlines = [] # [(Schema, SchemaAdminCls),]
@@ -457,11 +501,11 @@ class DocumentAdmin(SchemaAdmin):
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, ADDITION
+        object_id = '%s:%s' % (object._meta.collection, object.pk)
         LogEntry.objects.log_action(
             user_id         = request.user.pk,
-            #content_type_id = ContentType.objects.get_for_model(object).pk,
-            content_type_id = None,
-            object_id       = object.get_id(),
+            content_type_id = ContentType.objects.get_for_model(self.proxy_django_model).pk,
+            object_id       = object_id,
             object_repr     = force_unicode(object),
             action_flag     = ADDITION
         )
@@ -473,11 +517,11 @@ class DocumentAdmin(SchemaAdmin):
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, CHANGE
+        object_id = '%s:%s' % (object._meta.collection, object.pk)
         LogEntry.objects.log_action(
             user_id         = request.user.pk,
-            #content_type_id = ContentType.objects.get_for_model(object).pk,
-            content_type_id = None,
-            object_id       = object.get_id(),
+            content_type_id = ContentType.objects.get_for_model(self.proxy_django_model).pk,
+            object_id       = object_id,
             object_repr     = force_unicode(object),
             action_flag     = CHANGE,
             change_message  = message
@@ -491,28 +535,33 @@ class DocumentAdmin(SchemaAdmin):
         The default implementation creates an admin LogEntry object.
         """
         from django.contrib.admin.models import LogEntry, DELETION
+        object_id = '%s:%s' % (object._meta.collection, object.pk)
         LogEntry.objects.log_action(
             user_id         = request.user.pk,
-            #content_type_id = ContentType.objects.get_for_model(self.model).pk,
-            content_type_id = None,
-            object_id       = object.get_id(),
+            content_type_id = ContentType.objects.get_for_model(self.proxy_django_model).pk,
+            object_id       = object_id,
             object_repr     = object_repr,
             action_flag     = DELETION
         )
     
-    def get_base_breadcrumbs(self):
+    def get_base_breadcrumbs(self, top=False):
         admin_name = self.admin_site.name
         model_name = self.model._meta.verbose_name
         opts = self.model._meta
         breadcrumbs = [
             Breadcrumb('Home', ['%s:index' % admin_name]),
-            Breadcrumb(opts.app_label, ['%s:app_list' % admin_name, (self.app_name,), {}]),
-            Breadcrumb(opts.verbose_name_plural, ['%s:%s_changelist' % (admin_name, self.app_name)]),
+            Breadcrumb(opts.app_label, ['%s:app_list' % admin_name, (self.app_label,), {}]),
         ]
+        if top:
+            breadcrumbs.append(Breadcrumb(opts.verbose_name_plural))
+        else:
+            breadcrumbs.append(Breadcrumb(opts.verbose_name_plural, ['%s:%s_changelist' % (admin_name, self.app_name)]))
         return breadcrumbs
     
-    def get_instance_breadcrumb(self, obj=None):
+    def get_instance_breadcrumb(self, obj=None, include_link=False):
         if obj:
+            if include_link:
+                return Breadcrumb(unicode(obj), self.reverse(self.app_name+'_change', obj.pk))
             return Breadcrumb(unicode(obj))
         return Breadcrumb('Add %s' % self.model._meta.verbose_name)
     
