@@ -45,13 +45,18 @@ class InlineSchemaAdmin(SchemaAdmin):
             js.append('js/prepopulate.min.js')
         if self.filter_vertical or self.filter_horizontal:
             js.extend(['js/SelectBox.js' , 'js/SelectFilter2.js'])
-        media = forms.Media(js=['%s%s' % (settings.ADMIN_MEDIA_PREFIX, url) for url in js])
+        media = forms.Media(js=['%s%s' % (getattr(settings, 'ADMIN_MEDIA_PREFIX', 'admin'), url) for url in js])
         media.add_js(['%sadmin/js/primitivelist.js' % settings.STATIC_URL])
         media.add_css({'all': ['%sadmin/css/primitivelist.css' % settings.STATIC_URL]})
         return media
     media = property(_media)
-
-    def get_formset(self, request, obj=None, **kwargs):
+    '''
+    def formfield_for_field(self, *args, **kwargs):
+        ret = super(InlineSchemaAdmin, self).formfield_for_field(*args, **kwargs)
+        assert False
+        return ret
+    '''
+    def get_formset(self, request, view, obj=None, **kwargs):
         """Returns a BaseInlineFormSet class for use in admin add/change views."""
         if self.declared_fieldsets:
             fields = flatten_fieldsets(self.declared_fieldsets)
@@ -71,7 +76,7 @@ class InlineSchemaAdmin(SchemaAdmin):
             "formset": self.formset,
             "fields": fields,
             "exclude": exclude,
-            "formfield_callback": curry(self.formfield_for_field, request=request),
+            "formfield_callback": curry(self.formfield_for_field, request=request, view=view), #view=None
             "extra": self.extra,
             "max_num": self.max_num,
             "can_delete": self.can_delete,
@@ -79,13 +84,19 @@ class InlineSchemaAdmin(SchemaAdmin):
         }
         defaults.update(kwargs)
         return inlinedocumentformset_factory(self.model, self.dotpath, **defaults)
-
-    def get_fieldsets(self, request, obj=None):
-        if self.declared_fieldsets:
-            return self.declared_fieldsets
-        form = self.get_formset(request).form
-        fields = form.base_fields.keys() + list(self.get_readonly_fields(request, obj))
-        return [(None, {'fields': fields})]
+    
+    def get_excludes(self):
+        return list(self.exclude)
+    
+    def formfield_for_jsonfield(self, prop, field, view, **kwargs):
+        request = kwargs.pop('request', None)
+        from fields import DotPathField
+        field = DotPathField
+        kwargs['dotpath'] = self.dotpath
+        kwargs['params'] = request.GET.copy()
+        if view.next_dotpath(): #???
+            kwargs['required'] = False
+        return field(**kwargs)
 
 class StackedInline(InlineSchemaAdmin):
     template = 'admin/edit_inline/stacked.html'
