@@ -3,9 +3,11 @@ from django.utils.encoding import force_unicode
 from django.utils.translation import ugettext as _
 from django.utils.html import escape, escapejs
 from django.views.generic import TemplateView, View
+from django.contrib.contenttypes.models import ContentType
 
 from base import AdminViewMixin
 from breadcrumbs import Breadcrumb
+from objecttools import LinkObjectTool
 import helpers
 
 from dockit import views
@@ -16,6 +18,7 @@ from dockit.schema.schema import Schema
 
 from urllib import urlencode
 from urlparse import parse_qsl
+import inspect
 
 CALL_BACK = "" #TODO
 
@@ -37,7 +40,7 @@ class DocumentViewMixin(AdminViewMixin):
         if self.template_name:
             return [self.template_name]
         opts = self.document._meta
-        app_label = opts.app_label
+        app_label = opts.app_label.lower()
         object_name = opts.object_name.lower()
         return ['admin/%s/%s/%s.html' % (app_label, object_name, self.template_suffix),
                 'admin/%s/%s.html' % (app_label, self.template_suffix),
@@ -53,7 +56,7 @@ class DocumentViewMixin(AdminViewMixin):
         opts = self.document._meta
         obj = self.object
         context = AdminViewMixin.get_context_data(self, **kwargs)
-        context.update({'root_path': self.admin_site.root_path,
+        context.update({'root_path': getattr(self.admin_site, 'root_path', None),
                         'app_label': opts.app_label,
                         'opts': opts,
                         'module_name': force_unicode(opts.verbose_name_plural),
@@ -214,7 +217,7 @@ class FragmentViewMixin(BaseFragmentViewMixin):
         return self.admin.prepopulated_fields
     
     def get_formsets(self):
-        formsets_cls = self.admin.get_formsets(self.request, self.get_active_object())
+        formsets_cls = self.admin.get_formsets(self.request, obj=self.get_active_object(), view=self)
         
         prefixes = {}
         formsets = list()
@@ -287,7 +290,7 @@ class FragmentViewMixin(BaseFragmentViewMixin):
         if hasattr(self, 'object'):
             obj = self.object
         
-        context.update({'root_path': self.admin_site.root_path,
+        context.update({'root_path': getattr(self.admin_site, 'root_path', None),
                         'app_label': opts.app_label,
                         'opts': opts,
                         'original': obj,
@@ -333,7 +336,8 @@ class FragmentViewMixin(BaseFragmentViewMixin):
     
     def get_object_tools(self, context):
         object_tools = list()
-        for object_tool in self.admin.get_object_tools(self.request):
+        obj = getattr(self, 'object', None)
+        for object_tool in self.admin.get_object_tools(self.request, obj):
             object_tools.append(object_tool.render(self.request, context))
         return object_tools
     
@@ -357,7 +361,7 @@ class FragmentViewMixin(BaseFragmentViewMixin):
             class Meta:
                 document = self.temp_document
                 schema = self.get_schema()
-                form_field_callback = self.formfield_for_field
+                formfield_callback = self.formfield_for_field
                 dotpath = self.dotpath() or None
                 exclude = self.admin.get_excludes() + self.get_readonly_fields()
                 #TODO fix readonly field behavior
@@ -501,37 +505,53 @@ class FragmentViewMixin(BaseFragmentViewMixin):
         return HttpResponseRedirect(self.admin.reverse(self.admin.app_name+'_changelist'))
 
 class IndexView(DocumentViewMixin, views.ListView):
-    template_suffix = 'change_list'
+    template_suffix = 'schema_change_list'
     
     def get_changelist(self):
         if not hasattr(self, 'changelist'):
             changelist_cls = self.admin.get_changelist(self.request)
-            self.changelist = changelist_cls(request=self.request,
-                                        model=self.document,
-                                        list_display=self.admin.list_display,
-                                        list_display_links=self.admin.list_display_links,
-                                        list_filter=self.admin.list_filter,
-                                        date_hierarchy=self.admin.date_hierarchy,
-                                        search_fields=self.admin.search_fields,
-                                        list_select_related=self.admin.list_select_related,
-                                        list_per_page=self.admin.list_per_page,
-                                        list_editable=self.admin.list_editable,
-                                        model_admin=self.admin,)
+            kwargs = {'request':self.request,
+                      'model':self.document,
+                      'list_display':self.admin.list_display,
+                      'list_display_links':self.admin.list_display_links,
+                      'list_filter':self.admin.list_filter,
+                      'date_hierarchy':self.admin.date_hierarchy,
+                      'search_fields':self.admin.search_fields,
+                      'list_select_related':self.admin.list_select_related,
+                      'list_per_page':self.admin.list_per_page,
+                      'list_max_show_all':self.admin.list_max_show_all,
+                      'list_editable':self.admin.list_editable,
+                      'model_admin':self.admin,}
+            argspec = inspect.getargspec(changelist_cls.__init__)
+            for key in kwargs.keys():
+                if key not in argspec.args:
+                    del kwargs[key]
+            self.changelist = changelist_cls(**kwargs)
         return self.changelist
     
+    def get_breadcrumbs(self):
+        return self.admin.get_base_breadcrumbs(top=True)
+    
     def get_context_data(self, **kwargs):
         context = views.ListView.get_context_data(self, **kwargs)
         context.update(DocumentViewMixin.get_context_data(self, **kwargs))
         cl = self.get_changelist()
         context.update({'cl': cl,
                         'title': cl.title,
-                        'is_popup': cl.is_popup,})
-        
+                        'is_popup': cl.is_popup,
+                        'breadcrumbs': self.get_breadcrumbs(),})
+        context['object_tools'] = self.get_object_tools(context)
         return context
     
     def get_queryset(self):
         cl = self.get_changelist()
         return cl.get_query_set()
+    
+    def get_object_tools(self, context):
+        object_tools = list()
+        for object_tool in self.admin.get_object_tools(self.request, add=True):
+            object_tools.append(object_tool.render(self.request, context))
+        return object_tools
 
 class ListFieldIndexView(BaseFragmentViewMixin, views.DetailView):
     template_suffix = 'listfield_change_list'
@@ -546,19 +566,26 @@ class ListFieldIndexView(BaseFragmentViewMixin, views.DetailView):
             dotpath = self.dotpath()
             changelist_cls = self.get_changelist_class()
             #CONSIDER: schemaadmin should have these values
-            self.changelist = changelist_cls(request=self.request,
-                                        model=self.schema,
-                                        instance=instance,
-                                        dotpath=dotpath,
-                                        list_display=self.admin.list_display,
-                                        list_display_links=self.admin.list_display_links,
-                                        list_filter=self.admin.list_filter,
-                                        date_hierarchy=self.admin.date_hierarchy,
-                                        search_fields=self.admin.search_fields,
-                                        list_select_related=self.admin.list_select_related,
-                                        list_per_page=self.admin.list_per_page,
-                                        list_editable=self.admin.list_editable,
-                                        model_admin=self.admin,)
+            kwargs = {'request':self.request,
+                      'model':self.schema,
+                      'list_display':self.admin.list_display,
+                      'list_display_links':self.admin.list_display_links,
+                      'list_filter':self.admin.list_filter,
+                      'date_hierarchy':self.admin.date_hierarchy,
+                      'search_fields':self.admin.search_fields,
+                      'list_select_related':self.admin.list_select_related,
+                      'list_per_page':self.admin.list_per_page,
+                      'list_max_show_all':self.admin.list_max_show_all,
+                      'list_editable':self.admin.list_editable,
+                      'model_admin':self.admin,}
+            from changelist import ChangeList #hack for Django 1.3 & 1.4 compatibility
+            argspec = inspect.getargspec(ChangeList.__init__)
+            for key in kwargs.keys():
+                if key not in argspec.args:
+                    del kwargs[key]
+            kwargs.update({'instance':instance,
+                           'dotpath':dotpath,})
+            self.changelist = changelist_cls(**kwargs)
         return self.changelist
     
     def get_breadcrumbs(self):
@@ -737,17 +764,24 @@ class UpdateView(FragmentViewMixin, views.UpdateView):
         return context
     
     def form_valid(self, form):
+        #TODO construct a change message
         self.admin.log_change(self.request, self.object, '')
         return FragmentViewMixin.form_valid(self, form)
 
 class DeleteView(DocumentViewMixin, views.DetailView):
-    template_suffix = 'delete_selected_confirmation'
+    template_suffix = 'schema_delete_confirmation'
     title = _('Delete')
     key = 'delete'
     
+    def get_breadcrumbs(self):
+        breadcrumbs = DocumentViewMixin.get_breadcrumbs(self)
+        breadcrumbs.append(self.admin.get_instance_breadcrumb(self.object, include_link=True))
+        breadcrumbs.append(Breadcrumb(self.title))
+        return breadcrumbs
+    
     def get_context_data(self, **kwargs):
         context = views.DetailView.get_context_data(self, **kwargs)
-        #context.update(FragmentViewMixin.get_context_data(self, **kwargs))
+        context.update(DocumentViewMixin.get_context_data(self, **kwargs))
         #TODO add what will be deleted
         return context
     
@@ -756,9 +790,35 @@ class DeleteView(DocumentViewMixin, views.DetailView):
         object_repr = unicode(self.object)
         self.object.delete()
         self.admin.log_deletion(request, self.object, object_repr)
-        return HttpResponseRedirect(self.admin.reverse(self.admin.app_name+'_changelist'))
+        return HttpResponseRedirect(self.get_success_url())
+    
+    def get_success_url(self):
+        return self.admin.reverse(self.admin.app_name+'_changelist')
 
-class HistoryView(DocumentViewMixin, views.ListView):
+class HistoryView(DocumentViewMixin, views.DetailView):
+    template_suffix = 'schema_history'
     title = _('History')
     key = 'history'
+    
+    def get_breadcrumbs(self):
+        breadcrumbs = DocumentViewMixin.get_breadcrumbs(self)
+        breadcrumbs.append(self.admin.get_instance_breadcrumb(self.object, include_link=True))
+        breadcrumbs.append(Breadcrumb(self.title))
+        return breadcrumbs
+    
+    def get_object_tool(self, request, obj):
+        return LinkObjectTool(self.title, self.admin.reverse('%s_%s' % (self.admin.app_name, self.key), obj.pk), css_class='historylink')
+    
+    def get_context_data(self, **kwargs):
+        context = views.DetailView.get_context_data(self, **kwargs)
+        context.update(DocumentViewMixin.get_context_data(self, **kwargs))
+        context['action_list'] = self.get_action_list()
+        return context
+    
+    def get_action_list(self):
+        obj = self.object
+        from django.contrib.admin.models import LogEntry
+        content_type = ContentType.objects.get_for_model(self.admin.proxy_django_model).pk
+        object_id = '%s:%s' % (obj._meta.collection, obj.pk)
+        return LogEntry.objects.filter(object_id=object_id, content_type_id=content_type)
 
