@@ -4,12 +4,23 @@ try:
 except ImportError:
     from pymongo.objectid import ObjectId
 
-from dockit.backends.base import BaseDocumentStorage
+from dockit.backends.base import BaseDocumentStorage, BaseIndexStorage
 from dockit.backends.queryset import BaseDocumentQuery
 
-from django.conf import settings
+class ValuesResultClass(dict):
+    def __getitem__(self, key):
+        try:
+            return dict.__getitem__(self, key)
+        except KeyError:
+            if key == 'pk':
+                return dict.__getitem__(self, '_id')
+            raise
 
 class DocumentQuery(BaseDocumentQuery):
+    def __init__(self, query_index, backend=None):
+        super(DocumentQuery, self).__init__(query_index, backend)
+        assert isinstance(self.backend, MongoIndexStorage), str(type(self.backend))
+    
     def _build_params(self, include_indexes=False):
         params =  dict()
         for op in self.query_index.inclusions:
@@ -64,9 +75,12 @@ class DocumentQuery(BaseDocumentQuery):
             raise self.document.DoesNotExist
         return self.wrap(ret)
     
-    def values(self):
+    def values(self, *limit_to, **kwargs):
         params = self._build_params(include_indexes=True)
-        raise NotImplementedError
+        fields = limit_to or None
+        if params:
+            return self.collection.find(params, fields=fields, as_class=ValuesResultClass)
+        return self.collection.find(fields=fields, as_class=ValuesResultClass)
     
     def __len__(self):
         return self.queryset.count()
@@ -84,10 +98,7 @@ class DocumentQuery(BaseDocumentQuery):
         else:
             return self.wrap(self.queryset[val])
 
-class MongoDocumentStorage(BaseDocumentStorage):
-    name = "mongodb"
-    _indexers = dict() #TODO this should be automatic
-
+class MongoStorageMixin(object):
     def __init__(self, username=None, password=None, host=None, port=None, db=None, **kwargs):
         self.connection = Connection(kwargs.get('HOST', host), kwargs.get('PORT', port))
         self.db = self.connection[kwargs.get('DB', db)]
@@ -96,6 +107,38 @@ class MongoDocumentStorage(BaseDocumentStorage):
     
     def get_collection(self, collection):
         return self.db[collection]
+
+class MongoIndexStorage(BaseIndexStorage, MongoStorageMixin):
+    name = "mongodb"
+    _indexers = dict() #TODO this should be automatic
+    
+    def register_index(self, query_index):
+        query = self.get_query(query_index)
+        params = query._build_params(include_indexes=True)
+        for key in params.keys(): #TODO this is a hack
+            params[key] = 1
+        if params:
+            collection = query_index.document._meta.collection
+            try:
+                self.get_collection(collection).ensure_index(params, background=True)
+            except TypeError:
+                self.get_collection(collection).ensure_index(params.items(), background=True)
+    
+    def destroy_index(self, query_index):
+        pass #TODO
+    
+    def get_query(self, query_index):
+        return DocumentQuery(query_index, backend=self)
+    
+    def on_save(self, doc_class, collection, data, doc_id):
+        #CONSIDER supporting standalone mongo indexes
+        pass #no operation needed
+    
+    def on_delete(self, doc_class, collection, doc_id):
+        pass #no operation needed
+
+class MongoDocumentStorage(BaseDocumentStorage, MongoStorageMixin):
+    name = "mongodb"
     
     def save(self, doc_class, collection, data):
         id_field = self.get_id_field_name()
@@ -120,18 +163,6 @@ class MongoDocumentStorage(BaseDocumentStorage):
     def get_id_field_name(self):
         return '_id'
     
-    def register_index(self, query_index):
-        query = self.get_query(query_index)
-        params = query._build_params(include_indexes=True)
-        for key in params.keys(): #TODO this is a hack
-            params[key] = 1
-        if params:
-            collection = query_index.document._meta.collection
-            try:
-                self.get_collection(collection).ensure_index(params, background=True)
-            except TypeError:
-                self.get_collection(collection).ensure_index(params.items(), background=True)
-    
     def get_query(self, query_index):
         return DocumentQuery(query_index)
 
