@@ -2,8 +2,9 @@ class BaseDocumentQuery(object):
     """
     Implemented by the backend to execute a certain index
     """
-    def __init__(self, query_index):
+    def __init__(self, query_index, backend=None):
         self.query_index = query_index
+        self._backend = backend
     
     @property
     def document(self):
@@ -11,7 +12,9 @@ class BaseDocumentQuery(object):
     
     @property
     def backend(self):
-        return self.document._meta.get_backend()
+        if self._backend:
+            return self._backend
+        return self.document._meta.get_index_backend_for_read(self.query_index)
     
     def _get_indexer_for_operation(self, document, op):
         return self.backend._get_indexer_for_operation(document, op)
@@ -35,7 +38,7 @@ class BaseDocumentQuery(object):
     def get_from_filter_operations(self, filter_operations):
         raise NotImplementedError
     
-    def values(self):
+    def values(self, *limit_to, **kwargs):
         raise NotImplementedError
     
     def __getitem__(self, val):
@@ -46,11 +49,16 @@ class BaseDocumentQuery(object):
 
 class QuerySet(object):
     '''
-    Acts as the queryset level caching layer
+    Acts as a caching layer around an implemented `BaseDocumentQuery`
+    TODO: implement actual caching
     '''
     def __init__(self, query):
         self.query = query
     
+    @property
+    def document(self):
+        return self.query.document
+    
     def __len__(self):
         #TODO cache
         return self.query.__len__()
@@ -61,8 +69,9 @@ class QuerySet(object):
     def delete(self):
         return self.query.delete()
     
-    def values(self):
-        return self.query.values()
+    def values(self, *limit_to, **kwargs):
+        #TODO cache
+        return self.query.values(*limit_to, **kwargs)
     
     def get(self, **kwargs):
         #TODO cache
