@@ -1,11 +1,11 @@
 from django.forms.fields import ChoiceField, Field, EMPTY_VALUES
 from django.forms.widgets import HiddenInput, SelectMultiple, MultipleHiddenInput
 from django.forms import ValidationError
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 from django.utils.encoding import force_unicode, smart_unicode
 from django.utils import simplejson as json
 
-from widgets import PrimitiveListWidget
+from dockit.forms.widgets import PrimitiveListWidget
 
 import inspect
 
@@ -124,7 +124,7 @@ class SchemaChoiceIterator(object):
         return (self.field.prepare_value(obj), self.field.label_from_instance(obj))
 
 class SchemaChoiceField(ChoiceField):
-    """A ChoiceField whose choices are a model QuerySet."""
+    """A ChoiceField whose choices are a document QuerySet."""
     # This class is a subclass of ChoiceField for purity, but it doesn't
     # actually use any of ChoiceField's implementation.
     default_error_messages = {
