@@ -116,10 +116,14 @@ def document_to_dict(document, instance, properties=None, exclude=None, dotpath=
             continue
         if exclude and prop_name in exclude:
             continue
+        if prop_name in document._meta.fields:
+            prop = document._meta.fields[prop_name]
+            if not properties and not prop.editable:
+                continue
         data[prop_name] = src_data[prop_name]
     return data
 
-def fields_for_document(document, properties=None, exclude=None, form_field_callback=None, dotpath=None):
+def fields_for_document(document, properties=None, exclude=None, formfield_callback=None, dotpath=None):
     """
     Returns a ``SortedDict`` containing form fields for the given document.
 
@@ -151,12 +155,14 @@ def fields_for_document(document, properties=None, exclude=None, form_field_call
             continue
         if exclude and prop.name in exclude:
             continue
+        if not properties and not prop.editable:
+            continue
         
         field = prop.formfield()
         
-        if field and form_field_callback:
+        if field and formfield_callback:
             defaults = prop.formfield_kwargs()
-            field = form_field_callback(prop, type(field), **defaults)
+            field = formfield_callback(prop, type(field), **defaults)
         if field:
             field_list.append((prop.name, field))
     return SortedDict(field_list)
@@ -167,7 +173,7 @@ class DocumentFormOptions(object):
         self.schema = getattr(options, 'schema', None)
         self.properties = getattr(options, 'properties', None)
         self.exclude = getattr(options, 'exclude', None)
-        self.form_field_callback = getattr(options, 'form_field_callback', None)
+        self.formfield_callback = getattr(options, 'formfield_callback', None)
         self.dotpath = getattr(options, 'dotpath', None)
         
         #lookup the appropriate schema if none is given
@@ -201,7 +207,7 @@ class DocumentFormMetaClass(type):
                                                 'Meta', None))
         if opts.schema:
             fields = fields_for_document(opts.schema, opts.properties,
-                                         opts.exclude, form_field_callback=opts.form_field_callback,)
+                                         opts.exclude, formfield_callback=opts.formfield_callback,)
             # Override default docuemnt fields with any custom declared ones
             # (plus, include all the other declared fields).
             new_class.serialized_fields = fields.keys()
@@ -209,7 +215,7 @@ class DocumentFormMetaClass(type):
         elif opts.document: #TODO this should no longer be necessary
             # If a document is defined, extract form fields from it.
             fields = fields_for_document(opts.document, opts.properties,
-                                         opts.exclude, form_field_callback=opts.form_field_callback,
+                                         opts.exclude, formfield_callback=opts.formfield_callback,
                                          dotpath=opts.dotpath)
             # Override default docuemnt fields with any custom declared ones
             # (plus, include all the other declared fields).
@@ -221,7 +227,8 @@ class DocumentFormMetaClass(type):
         new_class.declared_fields = declared_fields
         new_class.base_fields = fields
         return new_class
-    
+
+#TODO call schema.full_clean()
 class BaseDocumentForm(BaseForm):
     """ Base Document Form object """
     
@@ -254,7 +261,8 @@ class BaseDocumentForm(BaseForm):
             value = field.widget.value_from_datadict(self.data, self.files, self.add_prefix(name))
             try:
                 arg_spec = inspect.getargspec(field.clean)
-                if len(arg_spec.args) > 2:
+                if ((hasattr(arg_spec, 'args') and len(arg_spec.args) > 2) or #py2.6+
+                     len(arg_spec[0]) > 2): #py2.5
                     initial = self.initial.get(name, field.initial)
                     if callable(initial):
                         initial = initial()
@@ -283,7 +291,7 @@ class BaseDocumentForm(BaseForm):
             if obj is None:
                 obj = opts.schema()
         else:
-            data = self.instance.to_primitive(self.instance)
+            data = self.instance.to_primitive(self.instance, generate_natural_key=False)
             obj = opts.schema.to_python(data)
         
         for prop_name in self.serialized_fields:
