@@ -1,12 +1,4 @@
-from exceptions import DotPathNotFound
-
-COLLECTIONS = dict()
-
-def register_collection(cls):
-    COLLECTIONS[cls._meta.collection] = cls
-
-def get_base_document(key):
-    return COLLECTIONS[key]
+from dockit.schema.exceptions import DotPathNotFound
 
 class UnSet(object):
     def __nonzero__(self):
@@ -21,7 +13,7 @@ class DotPathTraverser(object):
         self._called = False
     
     def resolve_for_schema(self, schema):
-        from fields import SchemaField
+        from dockit.schema.fields import SchemaField
         entry = {'value':None,
                  'field':SchemaField(schema=schema),
                  'part':None,}
@@ -29,13 +21,27 @@ class DotPathTraverser(object):
         self._resolve_loop()
     
     def resolve_for_instance(self, instance):
-        from fields import SchemaField
+        from dockit.schema.fields import SchemaField
         entry = {'value':instance,
                  'field':SchemaField(schema=type(instance)),
                  'part':None,}
         self.resolved_paths = [entry]
         self._resolve_loop()
     
+    def resolve_for_raw_data(self, data, schema=None):
+        from dockit.schema.fields import DictField, SchemaField
+        field = None
+        if schema:
+            field = SchemaField(schema=schema)
+            data = schema.to_python(data)
+        else:
+            field = DictField()
+        entry = {'value':data,
+                 'field':field,
+                 'part':None,}
+        self.resolved_paths = [entry]
+        self._resolve_loop()
+    
     def _resolve_loop(self):
         while not self._finished:
             self._called = False
@@ -71,6 +77,10 @@ class DotPathTraverser(object):
     def current_value(self):
         return self.current['value']
     
+    @property
+    def current_field(self):
+        return self.current['field']
+    
     def end(self, field=None, value=None):
         last_entry = self.resolved_paths[-1]
         if field:
@@ -84,8 +94,8 @@ class DotPathTraverser(object):
     def next(self, field=None, value=None):
         part = self.remaining_paths.pop(0)
         if value and field is None:
-            from fields import SchemaField
-            from schema import Schema
+            from dockit.schema.fields import SchemaField
+            from dockit.schema.schema import Schema
             if isinstance(value, Schema):
                 field = SchemaField(schema=type(value))
         entry = {'field':field,
@@ -160,3 +170,17 @@ class DotPathSet(set):
         else:
             self.add(value)
 
+class GenericDotPathObject(object):
+    traverse_types = [(list, DotPathList),
+                      (dict, DotPathDict),
+                      (set, DotPathSet),]
+    
+    def __init__(self, value):
+        self.value = value
+    
+    def traverse_dot_path(self, traverser):
+        for atype, acls in self.traverse_types:
+            if isinstance(self.value, atype):
+                return acls(self.value).traverse_dot_path(traverser)
+        return traverser.end(value=self.value)
+
