@@ -1,4 +1,4 @@
-from fields import BaseField as Field
+from dockit.schema.fields import BaseField as Field
 
 from django.db.models.fields.files import FieldFile, FileDescriptor
 from django.utils.encoding import force_unicode, smart_str
@@ -69,6 +69,10 @@ class FileField(Field):
             return val
         if getattr(val, 'storage_path', False):
             return val.storage_path
+        
+        if hasattr(val, 'open'): 
+            #rewind file to the begining
+            val.open()
         name = self.generate_filename(val.name)
         name = self.storage.save(name, val)
         assert name
@@ -81,3 +85,6 @@ class FileField(Field):
         ret.storage_path = val
         return ret
 
+class ImageField(FileField):
+    pass #TODO
+
